"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var Message_1 = require("../../Message");

var Symbols_1 = require("../../Symbols"); // a=1 a+=2


function assignmentExpressionHandler(node) {
  var _this = this; // var s = function(){}
  // s.name === s


  if (node.left.type === "Identifier" && node.right.type === "FunctionExpression" && !node.right.id) {
    node.right.id = {
      type: "Identifier",
      name: node.left.name
    };
  }

  var dataGetter = this.createLeftObjectGetter(node.left);
  var nameGetter = this.createNameGetter(node.left);
  var rightValueGetter = this.createClosure(node.right);
  return function () {
    var MArray = _this.globalScope.data['Array']; // dataGetter执行时，判断如果是const且已经初始化，会报错

    var data = dataGetter();
    var name = nameGetter();
    var realName;

    if (Symbols_1.isSymbol(name)) {
      realName = Symbols_1.storeKey(name);
    } else {
      realName = name;
    }

    var rightValue = rightValueGetter();

    if (node.operator !== "=") {
      // if a is undefined
      // a += 1
      _this.assertVariable(data, name, node);
    }

    switch (node.operator) {
      case "=":
        if (Symbols_1.isSymbol(name)) {
          Object.defineProperty(data, realName, {
            value: rightValue,
            writable: true,
            enumerable: false,
            configurable: true
          });
          return rightValue;
        } else {
          return updateValue(MArray, data, realName, rightValue); // return (data[realName] = rightValue);
        }

      case "+=":
        return updateValue(MArray, data, realName, data[realName] + rightValue);
      // return (data[realName] += rightValue);

      case "-=":
        return updateValue(MArray, data, realName, data[realName] - rightValue);
      // return (data[realName] -= rightValue);

      case "*=":
        return updateValue(MArray, data, realName, data[realName] * rightValue);
      // return (data[realName] *= rightValue);
      // case "**=":
      // data[name]: Getter may be triggered
      // 	return (data[name] = Math.pow(data[name], rightValue));

      case "/=":
        return updateValue(MArray, data, realName, data[realName] / rightValue);
      // return (data[realName] /= rightValue);

      case "%=":
        return updateValue(MArray, data, realName, data[realName] % rightValue);
      // return (data[realName] %= rightValue);

      case "<<=":
        return updateValue(MArray, data, realName, data[realName] << rightValue);
      // return (data[realName] <<= rightValue);

      case ">>=":
        return updateValue(MArray, data, realName, data[realName] >> rightValue);
      // return (data[realName] >>= rightValue);

      case ">>>=":
        return updateValue(MArray, data, realName, data[realName] >>> rightValue);
      // return (data[realName] >>>= rightValue);

      case "&=":
        return updateValue(MArray, data, realName, data[realName] & rightValue);
      // return (data[realName] &= rightValue);

      case "^=":
        return updateValue(MArray, data, realName, data[realName] ^ rightValue);
      // return (data[realName] ^= rightValue);

      case "|=":
        return updateValue(MArray, data, realName, data[realName] | rightValue);
      // return (data[realName] |= rightValue);

      default:
        throw _this.createInternalThrowError(Message_1.Messages.AssignmentExpressionSyntaxError, node.type, node);
    }
  };
}

exports.assignmentExpressionHandler = assignmentExpressionHandler;

function updateValue(MArray, data, name, value) {
  return data[name] = value;
}