"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var Message_1 = require("../../Message"); // let {name, ...rest} = obj
// let [first, ...rest] = arr


function objectPatternAssignHandler(node) {
  var _this = this; // var s = function(){}
  // s.name === s


  if (node.left.type === "Identifier" && node.right.type === "FunctionExpression" && !node.right.id) {
    node.right.id = {
      type: "Identifier",
      name: node.left.name
    };
  }

  var dataGetter = this.createLeftObjectGetter(node.left);
  var nameGetter = this.createNameGetter(node.left);
  var rightValueGetter = this.createClosure(node.right);
  var rightName = node.right.id ? node.right.id.name : 'rightValue';
  var rest = node.rest;
  var copyIndex = node.index;
  var arrRest = node.arrRest;
  return function () {
    // dataGetter执行时，判断如果是const且已经初始化，会报错
    var data = dataGetter();
    var name = nameGetter();
    var rightValue = rightValueGetter();

    if (copyIndex != null) {
      // ArrayPattern let [first, ...rest] = arr
      if (arrRest) {
        // let [...rest] = arr
        if (rightValue.slice && typeof rightValue.slice === 'function') {
          return data[name] = rightValue.slice(copyIndex);
        } else {
          throw _this.createInternalThrowError(Message_1.Messages.NormalError, rightValue + ".slice is not a function");
        }
      } else {
        // let [first, second] = arr
        return data[name] = rightValue[copyIndex];
      }
    } else {
      // ObjectPattern let {name, ...rest} = obj
      if (rest && rest.length) {
        if (!rightValue || typeof rightValue !== 'object') {
          return data[name] = {};
        }

        var res_1 = {};
        Object.getOwnPropertyNames(rightValue).forEach(function (key) {
          if (rest.indexOf(key) == -1) {
            res_1[key] = rightValue[key];
          }
        });
        return data[name] = res_1;
      } else {
        return data[name] = rightValue[name];
      }
    }
  };
}

exports.objectPatternAssignHandler = objectPatternAssignHandler;