"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var Symbols_1 = require("../../Symbols");

var TokenClass_1 = require("../../TokenClass");

var Scope_1 = require("../../Scope");

var hasOwnProperty = Object.prototype.hasOwnProperty; // try{...}catch(e){...}finally{}

function tryStatementHandler(node) {
  var _this = this;

  this.blockDeclareStart();
  var blockClosure = this.createClosure(node.block);
  var tryLexDecl = this.blockDeclareEnd();
  var handlerClosure = node.handler ? this.catchClauseHandler(node.handler) : null;
  this.blockDeclareStart();
  var finalizerClosure = node.finalizer ? this.createClosure(node.finalizer) : null;
  var finalLexDecl = this.blockDeclareEnd();
  return function () {
    var currentScope = _this.getCurrentScope();

    var currentContext = _this.getCurrentContext();

    var labelStack = currentScope.labelStack.concat([]);

    var callStack = _this.callStack.concat([]);

    var result = Symbols_1.EmptyStatementReturn;
    var finalReturn;
    var throwError;

    var reset = function reset() {
      _this.setCurrentScope(currentScope); //reset scope


      _this.setCurrentContext(currentContext); //reset context


      currentScope.labelStack = labelStack; //reset label stack

      _this.callStack = callStack; //reset call stack
    };
    /**
     * try{...}catch(e){...}finally{...} execution sequence:
     * try stmt
     * try throw
     * catch stmt (if)
     * finally stmt
     * finally throw or finally return
     * catch throw or catch return
     * try return
     */


    try {
      var prevScope = void 0;
      var newScope = void 0;

      if (tryLexDecl) {
        newScope = Scope_1.createScope(_this.getCurrentScope(), "BScope(try)", "block");
        newScope.lexDeclared = tryLexDecl;
        prevScope = _this.entryBlockScope(newScope);
      }

      result = _this.setValue(blockClosure());

      if (tryLexDecl) {
        _this.setCurrentContext(prevScope);
      }

      if (result instanceof TokenClass_1.Return) {
        finalReturn = result;
      }
    } catch (err) {
      reset();

      if (_this.isInterruptThrow(err)) {
        throw err;
      }

      if (handlerClosure) {
        try {
          result = _this.setValue(handlerClosure(err));

          if (result instanceof TokenClass_1.Return) {
            finalReturn = result;
          }
        } catch (err) {
          reset();

          if (_this.isInterruptThrow(err)) {
            throw err;
          } // save catch throw error


          throwError = err;
        }
      }
    } // finally {


    if (finalizerClosure) {
      try {
        var prevScope = void 0;
        var newScope = void 0;

        if (finalLexDecl) {
          newScope = Scope_1.createScope(_this.getCurrentScope(), "BScope(final)", "block");
          newScope.lexDeclared = finalLexDecl;
          prevScope = _this.entryBlockScope(newScope);
        } //do not save finally result


        result = finalizerClosure();

        if (finalLexDecl) {
          _this.setCurrentScope(prevScope);
        }

        if (result instanceof TokenClass_1.Return) {
          finalReturn = result;
        } // finalReturn = finalizerClosure();

      } catch (err) {
        reset();

        if (_this.isInterruptThrow(err)) {
          throw err;
        } // save finally throw error


        throwError = err;
      } // if (finalReturn instanceof Return) {
      // 	result = finalReturn;
      // }

    } // }


    if (throwError) throw throwError;

    if (finalReturn) {
      return finalReturn;
    }

    return result;
  };
}

exports.tryStatementHandler = tryStatementHandler; // ... catch(e){...}

function catchClauseHandler(node) {
  var _this = this;

  var paramNameGetter = this.createParamNameGetter(node.param); // catch中的代码会新开一个函数级别的作用域，
  // 但是不同的是context并没有变（函数调用的话，context会改变），

  var oldDeclVars = this.collectDeclVars;
  var oldDeclFuncs = this.collectDeclFuncs;
  var oldDeclLex = this.collectDeclLex; // 准备新scope的声明提升变量

  this.collectDeclVars = Object.create(null);
  this.collectDeclFuncs = Object.create(null);
  this.collectDeclLex = []; // set scope

  this.blockDeclareStart();
  var bodyClosure = this.createClosure(node.body);
  var lexDecl = this.blockDeclareEnd(); // 这里是准备好的变量和函数声明提升

  var declVars = this.collectDeclVars;
  var declFuncs = this.collectDeclFuncs;
  var declLex = this.collectDeclLex; // 恢复

  this.collectDeclVars = oldDeclVars;
  this.collectDeclFuncs = oldDeclFuncs;
  this.collectDeclLex = oldDeclLex;
  return function (err) {
    // get param name "e"
    var paramName = paramNameGetter();

    var prevScope = _this.getCurrentScope();

    var currentScope = Scope_1.createScope(prevScope, "FScope(catch)");
    currentScope.lexDeclared = lexDecl;

    _this.setCurrentScope(currentScope);

    _this.addDeclarationsToScope(declVars, declFuncs, currentScope); // add "e" to scope


    currentScope.data[paramName] = err;
    var result; // run

    result = bodyClosure();

    _this.setCurrentScope(prevScope);

    return result;
  };
}

exports.catchClauseHandler = catchClauseHandler;