"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var handler = require("./handler");

var ClosureHandler =
/** @class */
function () {
  function ClosureHandler() {
    this.binaryExpressionHandler = handler.binaryExpressionHandler;
    this.logicalExpressionHandler = handler.logicalExpressionHandler;
    this.unaryExpressionHandler = handler.unaryExpressionHandler;
    this.updateExpressionHandler = handler.updateExpressionHandler;
    this.objectExpressionHandler = handler.objectExpressionHandler;
    this.arrayExpressionHandler = handler.arrayExpressionHandler;
    this.callExpressionHandler = handler.callExpressionHandler;
    this.newExpressionHandler = handler.newExpressionHandler;
    this.memberExpressionHandler = handler.memberExpressionHandler;
    this.thisExpressionHandler = handler.thisExpressionHandler;
    this.sequenceExpressionHandler = handler.sequenceExpressionHandler;
    this.literalHandler = handler.literalHandler;
    this.identifierHandler = handler.identifierHandler;
    this.assignmentExpressionHandler = handler.assignmentExpressionHandler;
    this.functionDeclarationHandler = handler.functionDeclarationHandler;
    this.variableDeclarationHandler = handler.variableDeclarationHandler;
    this.programHandler = handler.programHandler;
    this.expressionStatementHandler = handler.expressionStatementHandler;
    this.emptyStatementHandler = handler.emptyStatementHandler;
    this.returnStatementHandler = handler.returnStatementHandler;
    this.functionExpressionHandler = handler.functionExpressionHandler;
    this.ifStatementHandler = handler.ifStatementHandler;
    this.conditionalExpressionHandler = handler.conditionalExpressionHandler;
    this.forStatementHandler = handler.forStatementHandler;
    this.whileStatementHandler = handler.whileStatementHandler;
    this.doWhileStatementHandler = handler.doWhileStatementHandler;
    this.forInStatementHandler = handler.forInStatementHandler;
    this.forOfStatementHandler = handler.forOfStatementHandler;
    this.withStatementHandler = handler.withStatementHandler;
    this.throwStatementHandler = handler.throwStatementHandler;
    this.tryStatementHandler = handler.tryStatementHandler;
    this.continueStatementHandler = handler.continueStatementHandler;
    this.breakStatementHandler = handler.breakStatementHandler;
    this.switchStatementHandler = handler.switchStatementHandler;
    this.labeledStatementHandler = handler.labeledStatementHandler;
    this.debuggerStatementHandler = handler.debuggerStatementHandler;
    this.catchClauseHandler = handler.catchClauseHandler;
    this.groupStatementHandler = handler.groupStatementHandler;
    this.arrowFunctionExpressionHandler = handler.arrowFunctionExpressionHandler;
    this.objectPatternAssignHandler = handler.objectPatternAssignHandler;
    this.spreadElementHandler = handler.spreadElementHandler;
    this.templateLiteralHandler = handler.templateLiteralHandler;
    this.classDeclarationHandler = handler.classDeclarationHandler;
    this.classExpressionHandler = handler.classExpressionHandler;
    this.JSXElementHandler = handler.JSXElementHandler;
    this.JSXMemberExpressionHandler = handler.JSXMemberExpressionHandler;
    this.JSXExpressionContainerHandler = handler.JSXExpressionContainerHandler;
    this.JSXTextHandler = handler.JSXTextHandler;
    this.JSXIdentifierHandler = handler.JSXIdentifierHandler;
    this.JSXAttributeHandler = handler.JSXAttributeHandler;
    this.JSXSpreadAttributeHandler = handler.JSXSpreadAttributeHandler;
    this.superHandler = handler.superHandler;
  }

  ClosureHandler.prototype.getClosure = function (node) {
    var closure = null;

    switch (node.type) {
      case 'ClassDeclaration':
        closure = this.classDeclarationHandler(node);
        break;

      case 'ClassExpression':
        closure = this.classExpressionHandler(node);
        break;

      case 'Super':
        closure = this.superHandler(node);
        break;

      case "BinaryExpression":
        closure = this.binaryExpressionHandler(node);
        break;

      case "LogicalExpression":
        closure = this.logicalExpressionHandler(node);
        break;

      case "UnaryExpression":
        closure = this.unaryExpressionHandler(node);
        break;

      case "UpdateExpression":
        closure = this.updateExpressionHandler(node);
        break;

      case "ObjectExpression":
        closure = this.objectExpressionHandler(node);
        break;

      case "ArrayExpression":
        closure = this.arrayExpressionHandler(node);
        break;

      case "CallExpression":
        closure = this.callExpressionHandler(node);
        break;

      case "NewExpression":
        closure = this.newExpressionHandler(node);
        break;

      case "MemberExpression":
        closure = this.memberExpressionHandler(node);
        break;

      case "ThisExpression":
        closure = this.thisExpressionHandler(node);
        break;

      case "SequenceExpression":
        closure = this.sequenceExpressionHandler(node);
        break;

      case "Literal":
        closure = this.literalHandler(node);
        break;

      case "Identifier":
        closure = this.identifierHandler(node);
        break;

      case "AssignmentExpression":
        closure = this.assignmentExpressionHandler(node);
        break;

      case "FunctionDeclaration":
        closure = this.functionDeclarationHandler(node);
        break;

      case 'ArrowFunctionExpression':
        closure = this.arrowFunctionExpressionHandler(node);
        break;

      case 'SpreadElement':
        closure = this.spreadElementHandler(node);
        break;

      case "VariableDeclaration":
        closure = this.variableDeclarationHandler(node);
        break;

      case 'TemplateLiteral':
        closure = this.templateLiteralHandler(node);
        break;

      case "BlockStatement":
      case "Program":
        closure = this.programHandler(node);
        break;
      // groupStatement 是为了编译需要自己加的，为了将若干表达式聚合起来一起执行，但是又不单独生成blockScope
      // @ts-ignore

      case "GroupStatement":
        closure = this.groupStatementHandler(node);
        break;
      // let {name, ...rest} = obj的赋值，写成了一个handler方便一点，要拆解开的话rest不太好实现
      // @ts-ignore

      case "ObjectPatternAssignExpression":
        closure = this.objectPatternAssignHandler(node);
        break;

      case "ExpressionStatement":
        closure = this.expressionStatementHandler(node);
        break;

      case "EmptyStatement":
        closure = this.emptyStatementHandler(node);
        break;

      case "ReturnStatement":
        closure = this.returnStatementHandler(node);
        break;

      case "FunctionExpression":
        closure = this.functionExpressionHandler(node);
        break;

      case "IfStatement":
        closure = this.ifStatementHandler(node);
        break;

      case "ConditionalExpression":
        closure = this.conditionalExpressionHandler(node);
        break;

      case "ForStatement":
        closure = this.forStatementHandler(node);
        break;

      case "WhileStatement":
        closure = this.whileStatementHandler(node);
        break;

      case "DoWhileStatement":
        closure = this.doWhileStatementHandler(node);
        break;

      case 'ForOfStatement':
        closure = this.forOfStatementHandler(node);
        break;

      case "ForInStatement":
        closure = this.forInStatementHandler(node);
        break;

      case "WithStatement":
        closure = this.withStatementHandler(node);
        break;

      case "ThrowStatement":
        closure = this.throwStatementHandler(node);
        break;

      case "TryStatement":
        closure = this.tryStatementHandler(node);
        break;

      case "ContinueStatement":
        closure = this.continueStatementHandler(node);
        break;

      case "BreakStatement":
        closure = this.breakStatementHandler(node);
        break;

      case "SwitchStatement":
        closure = this.switchStatementHandler(node);
        break;

      case "LabeledStatement":
        closure = this.labeledStatementHandler(node);
        break;

      case "DebuggerStatement":
        closure = this.debuggerStatementHandler(node);
        break;

      case "JSXElement":
        closure = this.JSXElementHandler(node);
        break;

      case "JSXExpressionContainer":
        closure = this.JSXExpressionContainerHandler(node);
        break;

      case "JSXIdentifier":
        closure = this.JSXIdentifierHandler(node);
        break;

      case "JSXAttribute":
        closure = this.JSXAttributeHandler(node);
        break;

      case "JSXSpreadAttribute":
        closure = this.JSXSpreadAttributeHandler(node);
        break;

      case "JSXMemberExpression":
        closure = this.JSXMemberExpressionHandler(node);
        break;

      case "JSXOpeningElement":
        // 在jsxelement就处理了，不可能到这里
        break;

      case "JSXText":
        closure = this.JSXTextHandler(node);
        break;

      case "JSXEmptyExpression":
        closure = function closure() {
          return null;
        };

        break;

      default:
        break;
    }

    return closure;
  };

  return ClosureHandler;
}();

exports.ClosureHandler = ClosureHandler;
applyMixins(ClosureHandler, handler);

function applyMixins(derivedCtor, father) {
  Object.getOwnPropertyNames(father).forEach(function (name) {
    derivedCtor.prototype[name] = father[name];
  });
}