"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
/**
 * 宏任务，供setTimeout和setInterval调用
 */

var MacroTask =
/** @class */
function () {
  function MacroTask(fn) {
    this._fn = fn;
  }

  MacroTask.prototype.run = function () {
    // need try-catch?
    this._fn();
  };

  return MacroTask;
}();

exports.MacroTask = MacroTask;

function noop() {}

var MicroTask =
/** @class */
function () {
  function MicroTask(fn) {
    this._fn = fn;
  }

  MicroTask.prototype.run = function () {
    this._fn();
  };

  return MicroTask;
}();

exports.MicroTask = MicroTask;

var MacroTaskList =
/** @class */
function () {
  /**
   *
   * @param maxLen 队列中允许存在的最大宏任务数量，防止过度使用导致运行时阻塞
   */
  function MacroTaskList(maxLen) {
    this.list = [];
    this.maxLen = maxLen || MacroTaskList.maxLen;
    this.microList = [];
    this.microFakeTop = null;
    this.macroRunning = null;
    this.macroNeedRun = null;
  }
  /**
   * 方法入小队
   * @param fn
   */


  MacroTaskList.prototype.microQueue = function (fn) {
    if (this.macroRunning) {
      this.microList.push(new MicroTask(fn));
      return;
    } else {
      var now = +new Date();

      if (!this.microFakeTop) {
        this.microFakeTop = new MacroTask(noop);

        if (this.list.length) {
          var time = this.list[0].time;
          this.list.unshift({
            task: this.microFakeTop,
            time: time < now ? time - 1 : now
          });
        } else {
          this.list.push({
            task: this.microFakeTop,
            time: now
          });
        }

        this.microList.push(new MicroTask(fn));
        this.nextRun(now);
      } else {
        this.microList.push(new MicroTask(fn));
      }
    }
  };
  /**
   * 方法入大队
   * @param fn
   * @param delay
   */


  MacroTaskList.prototype.queue = function (fn, delay) {
    delay = delay || 0;
    var task = new MacroTask(fn);
    var now = +new Date();
    var time = now + delay;

    if (!this.list.length) {
      this.list.push({
        time: time,
        task: task
      });
    } else {
      // todo: 二分查找，插入进去
      var targetIndex = void 0; // 永远插入到正在运行的后面，即使时间可能靠前

      var startIndex = this.macroNeedRun != null ? this.macroNeedRun.length : 0;

      for (var i = startIndex; i < this.list.length; i++) {
        if (time < this.list[i].time) {
          targetIndex = i;
          break;
        }
      }

      if (targetIndex == null) {
        targetIndex = this.list.length;
      }

      this.list.splice(targetIndex, 0, {
        time: time,
        task: task
      });
    }

    if (!this.macroRunning) {
      this.nextRun(now);
    }
  };

  MacroTaskList.prototype.distory = function () {
    if (this.nextTimer) {
      clearTimeout(this.nextTimer);
    }
  };

  MacroTaskList.prototype.findMacroToRun = function (now) {
    var arr = [];

    for (var i = 0; i < this.list.length; i++) {
      if (now >= this.list[i].time) {
        arr.push(this.list[i]);
      }
    }

    return arr;
  };
  /**
   * 执行到期的macro队列
   */


  MacroTaskList.prototype.flush = function () {
    if (this.macroRunning) {
      return;
    }

    if (!this.list.length) {
      return;
    }

    if (this.nextTimer) {
      clearTimeout(this.nextTimer);
    }

    var now = +new Date();
    var marcoToRun = this.findMacroToRun(now);

    if (!marcoToRun.length) {
      this.nextRun(now);
      return;
    }

    this.batchTasks(marcoToRun);
    now = +new Date();
    this.nextRun(now);
  };
  /**
   * 执行到期的macro任务，每执行完一个macro任务，清空一遍micro任务队列
   */


  MacroTaskList.prototype.batchTasks = function (marcoToRun) {
    this.microFakeTop = null;
    this.macroNeedRun = marcoToRun;
    var taskLen = marcoToRun.length;

    while (marcoToRun.length) {
      var item = marcoToRun.shift();
      this.macroRunning = item;
      item.task.run();

      while (this.microList.length) {
        var micro = this.microList.shift();
        micro.run();
      }
    } // 执行完毕，清理状态指针


    this.macroNeedRun = null;
    this.macroRunning = null; // 删除已经执行的任务

    this.list.splice(0, taskLen); // console.info('this.list is ', this.list, marcoToRun.length )
  };

  MacroTaskList.prototype.nextRun = function (now) {
    var _this = this;

    if (this.nextTimer) {
      clearTimeout(this.nextTimer);
    }

    if (!this.list.length) {
      return;
    }

    var delay = Math.max(0, this.list[0].time - now);
    this.nextTimer = setTimeout(function () {
      _this.nextTimer = null;

      _this.flush();
    }, delay);
  };

  MacroTaskList.maxLen = 9999;
  return MacroTaskList;
}();

exports.MacroTaskList = MacroTaskList;