"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var util_1 = require("../../../util");

var Symbols_1 = require("../../Symbols");

var TokenClass_1 = require("../../TokenClass");

var Scope_1 = require("../../Scope"); // for(var i = 0; i < 10; i++) {...}


function forStatementHandler(node) {
  var _this = this;

  var initClosure = util_1.noop;
  var testClosure = node.test ? this.createClosure(node.test) : function () {
    return true;
  };
  var updateClosure = util_1.noop;
  var initLexDeclared;

  var getBodyClosure = function getBodyClosure() {
    if (node.body.type == 'BlockStatement') {
      // node.body是个blockStatement，这里我们就不多此一举再去检测块作用域了。
      return {
        needBlock: null,
        closure: _this.createClosure(node.body)
      };
    } else {
      _this.blockDeclareStart();

      var closure = _this.createClosure(node.body);

      var bodyLex = _this.blockDeclareEnd();

      return {
        needBlock: bodyLex,
        closure: closure
      };
    }
  };

  if (node.type === "ForStatement") {
    if (node.init) {
      this.blockDeclareStart();
      initClosure = this.createClosure(node.init);
      initLexDeclared = this.blockDeclareEnd();
    }

    updateClosure = node.update ? this.createClosure(node.update) : util_1.noop;
  } else {// while, do-while
  }

  return function (pNode) {
    var labelName;
    var result = Symbols_1.EmptyStatementReturn;
    var shouldInitExec = node.type === "DoWhileStatement";

    if (pNode && pNode.type === "LabeledStatement") {
      labelName = pNode.label.name;
    }

    var prevScope;
    var newScope;

    if (initLexDeclared) {
      newScope = Scope_1.createScope(_this.getCurrentScope(), "BScope(for-let)", "block");
      newScope.lexDeclared = initLexDeclared;
      prevScope = _this.entryBlockScope(newScope);
    }

    for (initClosure(); shouldInitExec || testClosure(); updateClosure()) {
      shouldInitExec = false;
      var bodyClosure = getBodyClosure();
      var bodyPrev = void 0;
      var bodyScope = void 0;

      if (bodyClosure.needBlock) {
        bodyScope = Scope_1.createScope(_this.getCurrentScope(), "BScope(for-body)", "block");
        bodyScope.lexDeclared = bodyClosure.needBlock;
        bodyPrev = _this.entryBlockScope(bodyScope);
      } // save last value


      var ret = _this.setValue(bodyClosure.closure()); // 恢复作用域


      if (bodyClosure.needBlock) {
        _this.setCurrentScope(bodyPrev);
      } // notice: never return Break or Continue!


      if (ret === Symbols_1.EmptyStatementReturn || ret === Symbols_1.Continue) continue;

      if (ret === Symbols_1.Break) {
        break;
      }

      result = ret; // stop continue label

      if (result instanceof TokenClass_1.ContinueLabel && result.value === labelName) {
        result = Symbols_1.EmptyStatementReturn;
        continue;
      }

      if (result instanceof TokenClass_1.Return || result instanceof TokenClass_1.BreakLabel || result instanceof TokenClass_1.ContinueLabel) {
        break;
      }
    }

    if (initLexDeclared) {
      _this.setCurrentScope(prevScope);
    }

    return result;
  };
}

exports.forStatementHandler = forStatementHandler;