"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var Message_1 = require("../../Message");

var util_1 = require("../../../util");

var Symbols_1 = require("../../Symbols"); // var o = {a: 1, b: 's', get name(){}, set name(){}  ...}


function objectExpressionHandler(node) {
  var _this = this;

  var items = [];

  function getKey(keyNode) {
    if (keyNode.type === "Identifier") {
      // var o = {a:1}
      return keyNode.name;
    } else if (keyNode.type === "Literal") {
      // var o = {"a":1}
      return keyNode.value;
    } else {
      return this.throwError(Message_1.Messages.ObjectStructureSyntaxError, keyNode.type, keyNode);
    }
  } // collect value, getter, and/or setter.


  var properties = Object.create(null);
  var computedProperties = [];
  node.properties.forEach(function (property) {
    if (property.type == 'Property') {
      var kind = property.kind;

      if (!property.computed) {
        var key = getKey(property.key);

        if (!properties[key] || kind === "init") {
          properties[key] = {};
        }

        properties[key][kind] = _this.createClosure(property.value);
        items.push({
          key: key,
          property: property
        });
      } else {
        var keyClosure = _this.createClosure(property.key);

        computedProperties.push({
          keyClosure: keyClosure,
          kind: kind,
          valueClosure: _this.createClosure(property.value)
        });
      }
    } else if (property.type == 'SpreadElement') {
      // ts声明没有这个type，也是醉了
      items.push({
        // @ts-ignore
        spread: _this.createClosure(property.argument)
      });
    }
  });
  return function () {
    var result = {};
    var len = items.length;
    var MArray = _this.globalScope.data['Array'];

    var _loop_1 = function _loop_1(i) {
      var item = items[i];

      if (item.key != null) {
        // named property
        var key = item.key;
        var kinds = properties[key];
        var value = kinds.init ? kinds.init() : undefined;
        var getter = kinds.get ? kinds.get() : function () {};
        var setter = kinds.set ? kinds.set() : function (a) {};

        if ("set" in kinds || "get" in kinds) {
          var descriptor = {
            configurable: true,
            enumerable: true,
            get: getter,
            set: setter
          };
          Object.defineProperty(result, key, descriptor);
        } else {
          var property = item.property;
          var kind = property.kind; // set function.name
          // var d = { test(){} }
          // var d = { test: function(){} }

          if (property.key.type === "Identifier" && property.value.type === "FunctionExpression" && kind === "init" && !property.value.id) {
            util_1.defineFunctionName(value, property.key.name);
          }

          result[key] = value;
        }
      } else {
        // spread object
        var targetObj_1 = item.spread();

        if (targetObj_1 && Array.isArray(targetObj_1)) {
          for (var i_1 = 0; i_1 < targetObj_1.length; i_1++) {
            result[String(i_1)] = targetObj_1[i_1];
          }
        } else if (targetObj_1 && typeof targetObj_1 === 'object') {
          // 解构只解构本身的属性，且不会copy不能枚举的和setter，copy的getter也会转换为一个简单的property
          var keys = Object.getOwnPropertyNames(targetObj_1);
          keys.forEach(function (key) {
            result[key] = targetObj_1[key];
          });
        } else {
          return "continue";
        }
      }
    }; // 非computed属性。保证顺序


    for (var i = 0; i < len; i++) {
      _loop_1(i);
    }

    var prop = {};
    computedProperties.forEach(function (pr) {
      var key = pr.keyClosure();
      var isSb = Symbols_1.isSymbol(key);
      var name = isSb ? Symbols_1.storeKey(key) : key;

      if (!prop[name]) {
        prop[name] = {};
      }

      prop[name][pr.kind] = pr.valueClosure();
      prop[name]['symbol'] = isSb;
    });
    Object.getOwnPropertyNames(prop).forEach(function (name) {
      var item = prop[name];

      if ("set" in item || "get" in item) {
        var descriptor = {
          configurable: true,
          enumerable: item.symbol ? false : true,
          get: item.get || function () {},
          set: item.set || function (a) {}
        };
        Object.defineProperty(result, name, descriptor);
      } else {
        if (item.symbol) {
          Object.defineProperty(result, name, {
            value: item.init,
            writable: true,
            enumerable: false,
            configurable: true
          });
        } else {
          result[name] = item.init;
        }
      }
    });
    return result;
  };
}

exports.objectExpressionHandler = objectExpressionHandler;