"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Message_1 = require("../../Message");
var Scope_1 = require("../../Scope");
var Symbols_1 = require("../../Symbols");
var extendStatics = function (d, b) {
    extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b)
            if (b.hasOwnProperty(p))
                d[p] = b[p]; };
    return extendStatics(d, b);
};
function __extend(child, father) {
    extendStatics(child, father);
    function __() { this.constructor = father; }
    child.prototype = father === null ? Object.create(father) : (__.prototype = father.prototype, new __());
}
;
function classExpressionHandler(node) {
    var _this_1 = this;
    var className = node.id ? node.id.name : ""; /**anonymous*/
    // @ts-ignore
    var classDecl = {
        static: [],
        fieldsArrow: [],
        fieldsProperty: [],
        method: [],
    };
    var superClass = node.superClass ? this.createClosure(node.superClass) : null;
    node.body.body.forEach(function (item) {
        if (item.type === 'MethodDefinition') {
            // 关注3个属性：kind\static\computed
            if (item.kind === 'constructor') {
                classDecl.cons = _this_1.createClosure(item.value);
            }
            else if (item.kind === 'method') {
                classDecl[item.static ? 'static' : 'method'].push({
                    name: {
                        computed: item.computed,
                        // @ts-ignore
                        value: item.computed ? _this_1.createClosure(item.key) : item.key.name
                    },
                    value: _this_1.createClosure(item.value)
                });
            }
            else if (item.kind === 'get' || item.kind === 'set') {
                // 类的setter和getter不支持
                throw _this_1.createInternalThrowError(Message_1.Messages.NormalError, 'not support getter and setter in class', node);
            }
        }
        else if (item.type === 'FieldDefinition') {
            // 关注static\computed
            if (item.static) {
                classDecl.static.push({
                    name: {
                        computed: item.computed,
                        // @ts-ignore
                        value: item.computed ? _this_1.createClosure(item.key) : item.key.name
                    },
                    value: _this_1.createClosure(item.value)
                });
            }
            else {
                // 如果是箭头函数，需要特殊自绑定，其余的（属性或者function函数），不用做处理
                // @ts-ignore
                var t = item.value.type == 'ArrowFunctionExpression' ? 'fieldsArrow' : 'fieldsProperty';
                classDecl[t].push({
                    name: {
                        computed: item.computed,
                        // @ts-ignore
                        value: item.computed ? _this_1.createClosure(item.key) : item.key.name
                    },
                    value: _this_1.createClosure(item.value)
                });
            }
        }
        else {
            throw _this_1.createInternalThrowError(Message_1.Messages.NormalError, 'unknown class body type ' + item.type, node.body);
        }
    });
    return function () {
        var self = _this_1;
        var _super = superClass ? superClass() : null;
        var cons;
        // 如果存在父类，并且有显示的constructor声明时，构建构造方法的时候要注入super变量
        if (_super && classDecl.cons) {
            var newScope = Scope_1.createScope(_this_1.getCurrentScope(), "FScope(constructor)", 'block');
            newScope.lexDeclared = {
                super: {
                    kind: 'const',
                    init: true
                }
            };
            newScope.data['super'] = _super;
            var prevScope = _this_1.entryBlockScope(newScope);
            cons = classDecl.cons();
            _this_1.setCurrentScope(prevScope);
        }
        else {
            cons = classDecl.cons ? classDecl.cons() : null;
        }
        var func = function () {
            var _this = this;
            if (superClass && !cons) {
                _this = _super.call(_this) || _this;
            }
            // 先绑定field属性，再执行constructor
            classDecl.fieldsArrow.forEach(function (item) {
                // @ts-ignore
                var prev = self.getCurrentContext();
                // @ts-ignore
                self.setCurrentContext(_this);
                var fn = item.value();
                // @ts-ignore
                self.setCurrentContext(prev);
                setKeyVal(_this, item, fn);
                // _this[item.name.computed?(item.name.value as BaseClosure)():item.name.value] = fn
            });
            classDecl.fieldsProperty.forEach(function (item) {
                setKeyVal(_this, item, item.value());
                // _this[item.name.computed?(item.name.value as BaseClosure)():item.name.value] = item.value()
            });
            if (cons) {
                cons.apply(_this, arguments);
            }
            return _this;
        };
        superClass && __extend(func, _super);
        classDecl.method.forEach(function (item) {
            func.prototype[item.name.computed ? item.name.value() : item.name.value] = item.value();
        });
        classDecl.static.forEach(function (item) {
            func[item.name.computed ? item.name.value() : item.name.value] = item.value();
        });
        if (className) {
            Object.defineProperty(func, "name", {
                value: className,
                writable: false,
                enumerable: false,
                configurable: true,
            });
        }
        return func;
    };
}
exports.classExpressionHandler = classExpressionHandler;
function setKeyVal(_this, item, val) {
    var keyval;
    var sbl = false;
    if (item.name.computed) {
        var t = item.name.value();
        if (Symbols_1.isSymbol(t)) {
            sbl = true;
            keyval = Symbols_1.storeKey(t);
        }
        else {
            keyval = t;
        }
    }
    else {
        keyval = item.name.value;
    }
    if (sbl) {
        Object.defineProperty(_this, keyval, {
            value: val,
            writable: true,
            enumerable: false,
            configurable: true,
        });
    }
    else {
        _this[keyval] = val;
    }
}
