"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Message_1 = require("../../Message");
var Symbols_1 = require("../../Symbols");
var TokenClass_1 = require("../../TokenClass");
var Scope_1 = require("../../Scope");
// for(x of xx){}
function forOfStatementHandler(node) {
    var _this = this;
    // for( k of obj) or for(o.k of obj) ...
    var left = node.left;
    var rightClosure = this.createClosure(node.right);
    var getBodyClosure = function () {
        if (node.body.type == 'BlockStatement') {
            // node.body是个blockStatement，这里我们就不多此一举再去检测块作用域了。
            return {
                needBlock: null,
                closure: _this.createClosure(node.body)
            };
        }
        else {
            _this.blockDeclareStart();
            var closure = _this.createClosure(node.body);
            var bodyLex = _this.blockDeclareEnd();
            return {
                needBlock: bodyLex,
                closure: closure,
            };
        }
    };
    // for-of的逻辑是：1、先检查有没有Symbol.iterator方法，有的话，调用获取迭代器
    // 2、如果没有检查是否是Array，是的话for循环取元素
    // 3、以上都不是，报错。例如传一个obj进来，是要报错的
    // for(let k of obj) {...}
    var initClosure;
    var initLexDecl;
    if (node.left.type === "VariableDeclaration") {
        this.blockDeclareStart();
        initClosure = this.createClosure(node.left);
        initLexDecl = this.blockDeclareEnd();
        if (!initLexDecl) {
            // 初始化参数是var，直接初始化即可
            initClosure();
            initClosure = null;
        }
        else {
            // 初始化参数是let，等到执行时创建scope后再初始化
        }
        // reset left
        left = node.left.declarations[0].id;
    }
    else {
    }
    return function (pNode) {
        var labelName;
        var result = Symbols_1.EmptyStatementReturn;
        if (pNode && pNode.type === "LabeledStatement") {
            labelName = pNode.label.name;
        }
        var Smbl = _this.globalScope.data['Symbol'];
        var MArray = _this.globalScope.data['Array'];
        var prevScope;
        var newScope;
        if (initLexDecl) {
            newScope = Scope_1.createScope(_this.getCurrentScope(), "BScope(forof-let)", "block");
            newScope.lexDeclared = initLexDecl;
            prevScope = _this.entryBlockScope(newScope);
        }
        var data = rightClosure();
        if (data[Symbols_1.storeKey(Smbl.iterator)] && typeof data[Symbols_1.storeKey(Smbl.iterator)] == 'function') {
            var iter = data[Symbols_1.storeKey(Smbl.iterator)]();
            var val = iter.next();
            while (val && !val.done) {
                var t = loopBody.call(_this, function (_) { result = _; }, initLexDecl, newScope, node, left, labelName, getBodyClosure, val.value);
                if (t === 'break') {
                    break;
                }
                val = iter.next();
            }
        }
        else if (Array.isArray(data)) {
            for (var i = 0; i < data.length; i++) {
                var t = loopBody.call(_this, function (_) { result = _; }, initLexDecl, newScope, node, left, labelName, getBodyClosure, data[i]);
                if (t === 'break') {
                    break;
                }
            }
        }
        else {
            throw _this.createInternalThrowError(Message_1.Messages.VariableNotIterableError, '', node);
        }
        // 恢复
        if (initLexDecl) {
            _this.setCurrentScope(prevScope);
        }
        return result;
    };
}
exports.forOfStatementHandler = forOfStatementHandler;
function loopBody(setResult, initLexDecl, newScope, node, left, labelName, getBodyClosure, rightVal) {
    if (initLexDecl) {
        // 将所有声明变量的初始化改为false，重新赋值
        Object.getOwnPropertyNames(newScope.lexDeclared).forEach(function (name) {
            newScope.lexDeclared[name].init = false;
        });
        this.variableDeclarationHandler({
            type: "VariableDeclaration",
            kind: node.left.kind,
            declarations: [{
                    type: "VariableDeclarator",
                    id: node.left.declarations[0].id,
                    init: {
                        type: "Literal",
                        value: rightVal,
                    }
                }]
        })();
    }
    else {
        // assign left to scope
        // k = x
        // o.k = x
        // 对于迭代变量的赋值应该在bodyScope之外
        this.assignmentExpressionHandler({
            type: "AssignmentExpression",
            operator: "=",
            left: left,
            right: {
                type: "Literal",
                value: rightVal,
            },
        })();
    }
    var bodyClosure = getBodyClosure();
    var bodyPrev;
    var bodyScope;
    // 对于for循环，每次执行body都需要绑定新的scope，这么来说确实效率会慢很多
    if (bodyClosure.needBlock) {
        bodyScope = Scope_1.createScope(this.getCurrentScope(), 'BScope(forin-body)', 'block');
        bodyScope.lexDeclared = bodyClosure.needBlock;
        bodyPrev = this.entryBlockScope(bodyScope);
    }
    // save last value
    var ret = this.setValue(bodyClosure.closure());
    // 恢复
    if (bodyClosure.needBlock) {
        this.setCurrentScope(bodyPrev);
    }
    // notice: never return Break or Continue!
    if (ret === Symbols_1.EmptyStatementReturn || ret === Symbols_1.Continue)
        return 'continue';
    if (ret === Symbols_1.Break) {
        return 'break';
    }
    setResult(ret);
    // stop continue label
    if (ret instanceof TokenClass_1.ContinueLabel && ret.value === labelName) {
        setResult(Symbols_1.EmptyStatementReturn);
        return 'continue';
    }
    if (ret instanceof TokenClass_1.Return ||
        ret instanceof TokenClass_1.BreakLabel ||
        ret instanceof TokenClass_1.ContinueLabel) {
        return 'break';
    }
    return;
}
