"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Scope_1 = require("../../Scope");
var TokenClass_1 = require("../../TokenClass");
var util_1 = require("../../../util");
// var f = function() {...}
// 在es5的标准下，没有块级作用域，只会在function下会创建一个新的scope。所以这里的操作和evaluate的感觉差不多
function functionExpressionHandler(node) {
    var _this = this;
    var self = this;
    var source = this.source;
    var oldDeclVars = this.collectDeclVars;
    var oldDeclFuncs = this.collectDeclFuncs;
    var oldDeclLex = this.collectDeclLex;
    // 准备新scope的声明提升变量
    this.collectDeclVars = Object.create(null);
    this.collectDeclFuncs = Object.create(null);
    this.collectDeclLex = [];
    var name = node.id ? node.id.name : ""; /**anonymous*/
    // 可变参数不计入function.length
    var paramLength = node.params.filter(function (_) { return _.type != 'RestElement'; }).length;
    var paramsGetter = node.params.map(function (param) { return ({
        type: param.type,
        closure: _this.createParamNameGetter(param)
    }); });
    this.blockDeclareStart();
    // set scope
    var bodyClosure = this.createClosure(node.body);
    var lexDecls = this.blockDeclareEnd();
    // 这里是准备好的变量和函数声明提升
    var declVars = this.collectDeclVars;
    var declFuncs = this.collectDeclFuncs;
    var declLex = this.collectDeclLex;
    this.collectDeclVars = oldDeclVars;
    this.collectDeclFuncs = oldDeclFuncs;
    this.collectDeclLex = oldDeclLex;
    // 创建一个新的scope，然后返回一个函数，该函数会在新建的scope执行
    // @ts-ignore
    return function (scope) {
        // bind current scope
        // 注意一个函数执行时候的作用域，并不是真正调用时候的作用域，而是声明时候代码所在地点的作用域
        // 函数名声明的作用域，等同于var functionName 声明所在的作用域
        var runtimeScope = scope || self.getCurrentScope();
        var func = function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            // @ts-ignore
            self.callStack.push("" + name);
            // @ts-ignore
            var prevScope = self.getCurrentScope();
            // @ts-ignore
            var prev_functionVarScope = self._functionVarScope;
            // 函数执行时，创建新的scope，然后下一行将程序的运行指针指向新的scope
            var currentScope = Scope_1.createScope(runtimeScope, "FunctionScope(" + name + ")");
            currentScope.lexDeclared = lexDecls;
            // @ts-ignore
            self.setCurrentScope(currentScope);
            // console.info('the current scope is ', currentScope)
            // 将准备好的变量和函数声明赋值到新的scope
            // @ts-ignore
            self.addDeclarationsToScope(declVars, declFuncs, currentScope);
            // @ts-ignore
            self._functionVarScope = currentScope;
            // var t = function(){ typeof t } // function
            // t = function(){ typeof t } // function
            // z = function tx(){ typeof tx } // function
            // but
            // d = { say: function(){ typeof say } } // undefined
            if (name) {
                currentScope.data[name] = func;
            }
            // init arguments var
            currentScope.data["arguments"] = arguments;
            paramsGetter.forEach(function (getter, i) {
                if (getter.type === 'RestElement') {
                    currentScope.data[getter.closure()] = args.slice(i);
                }
                else {
                    currentScope.data[getter.closure()] = args[i];
                }
            });
            // init this
            // @ts-ignore
            var prevContext = self.getCurrentContext();
            //for ThisExpression
            // @ts-ignore
            self.setCurrentContext(this);
            // 执行
            var result = bodyClosure();
            // 恢复
            // @ts-ignore
            self.setCurrentContext(prevContext);
            // @ts-ignore
            self.setCurrentScope(prevScope);
            // @ts-ignore
            self._functionVarScope = prev_functionVarScope;
            // @ts-ignore
            self.callStack.pop();
            if (result instanceof TokenClass_1.Return) {
                return result.value;
            }
        };
        util_1.defineFunctionName(func, name);
        Object.defineProperty(func, "length", {
            value: paramLength,
            writable: false,
            enumerable: false,
            configurable: true,
        });
        Object.defineProperty(func, "toString", {
            value: function () {
                return source.slice(node.start, node.end);
            },
            writable: true,
            configurable: true,
            enumerable: false,
        });
        Object.defineProperty(func, "valueOf", {
            value: function () {
                return source.slice(node.start, node.end);
            },
            writable: true,
            configurable: true,
            enumerable: false,
        });
        return func;
    };
}
exports.functionExpressionHandler = functionExpressionHandler;
