"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Symbols_1 = require("../../Symbols");
var TokenClass_1 = require("../../TokenClass");
var Scope_1 = require("../../Scope");
function switchStatementHandler(node) {
    var _this = this;
    var discriminantClosure = this.createClosure(node.discriminant);
    this.blockDeclareStart();
    var caseClosures = node.cases.map(function (item) {
        var testClosure = item.test ? _this.createClosure(item.test) : function () { return Symbols_1.DefaultCase; };
        var bodyClosure = _this.createClosure({
            // @ts-ignore
            type: "GroupStatement", body: item.consequent,
        });
        return function () { return ({
            testClosure: testClosure,
            bodyClosure: bodyClosure,
        }); };
    });
    var blockLexDecl = this.blockDeclareEnd();
    return function () {
        var value = discriminantClosure();
        var match = false;
        var result;
        var ret, defaultCase;
        // switch执行的时候共用同一个blockscope
        var prevScope;
        var newScope;
        if (blockLexDecl) {
            newScope = Scope_1.createScope(_this.getCurrentScope(), "BScope(switch)", "block");
            newScope.lexDeclared = blockLexDecl;
            prevScope = _this.entryBlockScope(newScope);
        }
        for (var i = 0; i < caseClosures.length; i++) {
            var item = caseClosures[i]();
            var test_1 = item.testClosure();
            if (test_1 === Symbols_1.DefaultCase) {
                defaultCase = item;
                continue;
            }
            if (match || test_1 === value) {
                match = true;
                ret = _this.setValue(item.bodyClosure());
                // notice: never return Break!
                if (ret === Symbols_1.EmptyStatementReturn)
                    continue;
                if (ret === Symbols_1.Break) {
                    break;
                }
                result = ret;
                if (result instanceof TokenClass_1.Return ||
                    result instanceof TokenClass_1.BreakLabel ||
                    result instanceof TokenClass_1.ContinueLabel ||
                    result === Symbols_1.Continue) {
                    break;
                }
            }
        }
        if (!match && defaultCase) {
            ret = _this.setValue(defaultCase.bodyClosure());
            var isEBC = ret === Symbols_1.EmptyStatementReturn || ret === Symbols_1.Break || ret === Symbols_1.Continue;
            // notice: never return Break or Continue!
            if (!isEBC) {
                result = ret;
            }
        }
        if (blockLexDecl) {
            _this.setCurrentScope(prevScope);
        }
        return result;
    };
}
exports.switchStatementHandler = switchStatementHandler;
