"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Message_1 = require("../../Message");
var Symbols_1 = require("../../Symbols");
function detectVaiable(node, varName, blockVariables) {
    if (node.kind == 'var') {
        /**
         * 如果var声明的scope中已经有同名的词法作用域的变量，这个var的声明需要报错：
         * Cannot redeclare block-scoped variable 'xxx'
         */
        if (this.collectDeclLex.some(function (_) { return _[varName]; })) {
            throw this.createInternalThrowError(Message_1.Messages.RedeclareBlockScopeVariableError, node.type, node);
        }
        // 如果是var，声明提升
        this.varDeclaration(varName);
    }
    else {
        // 如果是let\const声明，变量名加入一个特殊的队列collectDeclLex的栈顶hash
        var stackTop = this.collectDeclLex[this.collectDeclLex.length - 1];
        // collectDeclLex主要是在编译阶段收集块变量用，如果在执行阶段动态去声明一个变量，那collectDeclLex将是空数组，所以判断一下
        stackTop && (stackTop[varName] = {
            init: false,
            kind: node.kind
        });
        blockVariables.push(varName);
    }
}
// var i;
// var i=1;
// let i;
function variableDeclarationHandler(node) {
    var _this = this;
    var assignmentsClosure;
    var assignments = [];
    var blockVariables = [];
    var _loop_1 = function (i) {
        var decl = node.declarations[i];
        if (decl.id.type == 'ObjectPattern') {
            if (!decl.init) {
                throw this_1.createInternalThrowError(Message_1.Messages.SpreadPatternVariableNoInit, '', node);
            }
            // let {name, ...rest} = obj
            // 实际是声明了两个变量
            var properties = decl.id.properties;
            var alreadyVars_1 = [];
            properties.forEach(function (item) {
                if (item.type === 'Property') {
                    var varName = _this.getVariableName(item.value);
                    alreadyVars_1.push(varName);
                    // 检测一下是否块变量
                    detectVaiable.call(_this, node, varName, blockVariables);
                    assignments.push({
                        type: 'ObjectPatternAssignExpression',
                        left: item.value,
                        right: decl.init,
                    });
                }
                else if (item.type === 'RestElement') {
                    // @ts-ignore
                    var varName = item.argument.name;
                    detectVaiable.call(_this, node, varName, blockVariables);
                    assignments.push({
                        type: "ObjectPatternAssignExpression",
                        // @ts-ignore
                        left: item.argument,
                        right: decl.init,
                        rest: alreadyVars_1,
                    });
                }
                else {
                    throw _this.createInternalThrowError(Message_1.Messages.UnknownVariableDeclTypeError, '', node);
                }
            });
        }
        else if (decl.id.type == 'ArrayPattern') {
            // let [first, ...rest] = arr;
            if (!decl.init) {
                throw this_1.createInternalThrowError(Message_1.Messages.SpreadPatternVariableNoInit, '', node);
            }
            var elements = decl.id.elements;
            var alreadyVars_2 = [];
            elements.forEach(function (item, arr_ind) {
                if (item.type === 'Identifier') {
                    var varName = item.name;
                    alreadyVars_2.push(varName);
                    // 检测一下是否块变量
                    detectVaiable.call(_this, node, varName, blockVariables);
                    assignments.push({
                        type: 'ObjectPatternAssignExpression',
                        left: item,
                        right: decl.init,
                        index: arr_ind,
                    });
                }
                else if (item.type === 'RestElement') {
                    // @ts-ignore
                    var varName = item.argument.name;
                    detectVaiable.call(_this, node, varName, blockVariables);
                    assignments.push({
                        type: "ObjectPatternAssignExpression",
                        // @ts-ignore
                        left: item.argument,
                        right: decl.init,
                        index: arr_ind,
                        arrRest: true,
                    });
                }
                else {
                    throw _this.createInternalThrowError(Message_1.Messages.UnknownVariableDeclTypeError, '', node);
                }
            });
        }
        else {
            var variableName = this_1.getVariableName(decl.id);
            detectVaiable.call(this_1, node, variableName, blockVariables);
            // 一些情形，低于const的声明不会有init，比如for(const i in arr){}
            // 我们这里不做强校验，主动声明的const必须初始化，acorn在生成ast之前就有校验，我们信任acorn吧。另外，即使这里不初始化，之后对const的赋值也会报错，所以不会产生太大影响。
            // if(node.kind == 'const' && !decl.init){
            //     throw this.createInternalThrowError(Messages.ConstNotInitError, variableName, node)
            // }
            if (decl.init) {
                assignments.push({
                    type: "AssignmentExpression",
                    operator: "=",
                    left: decl.id,
                    right: decl.init,
                });
            }
        }
    };
    var this_1 = this;
    for (var i = 0; i < node.declarations.length; i++) {
        _loop_1(i);
    }
    if (assignments.length) {
        assignmentsClosure = this.createClosure({
            // @ts-ignore
            type: "GroupStatement",
            // @ts-ignore
            body: assignments,
        });
    }
    return function () {
        if (assignmentsClosure) {
            if (blockVariables.length) {
                var scope_1 = _this.getCurrentScope();
                // 确认初始化块级变量
                blockVariables.forEach(function (name) {
                    if (scope_1.lexDeclared[name].kind === 'const') {
                        // 对于const变量，我们只在复制运行的时候将init设为true，
                        // init设为true后，之后对该变量的所有左值访问都将报错
                    }
                    else if (scope_1.lexDeclared[name].kind === 'let') {
                        // 对于let变量，我们在这里讲init设为true就好，时机不太影响
                        scope_1.lexDeclared[name].init = true;
                    }
                });
            }
            var oldValue = _this.isVarDeclMode;
            _this.isVarDeclMode = true;
            assignmentsClosure();
            // console.info('oldvalue is ', oldValue)
            _this.isVarDeclMode = oldValue;
        }
        return Symbols_1.EmptyStatementReturn;
    };
}
exports.variableDeclarationHandler = variableDeclarationHandler;
